@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -455,8 +455,8 @@ public final class AllFetch {
     static final Comparator<MailMessage> DESC_COMP = new Comparator<MailMessage>() {
 
         public int compare(final MailMessage m1, final MailMessage m2) {
-            final Date d1 = m1.getReceivedDate();
-            final Date d2 = m2.getReceivedDate();
+            final Date d1 = m1.getReceivedDateDirect();
+            final Date d2 = m2.getReceivedDateDirect();
             final Integer refComp = compareReferences(d1, d2);
             return (refComp == null ? d1.compareTo(d2) : refComp.intValue()) * (-1);
         }
@@ -472,17 +472,10 @@ public final class AllFetch {
      *         references are <code>null</code>, or returns <code>null</code> if both references are not <code>null</code>
      */
     static Integer compareReferences(final Object o1, final Object o2) {
-        if ((o1 == null) && (o2 != null)) {
-            return Integer.valueOf(-1);
-        } else if ((o1 != null) && (o2 == null)) {
-            return Integer.valueOf(1);
-        } else if ((o1 == null) && (o2 == null)) {
-            return Integer.valueOf(0);
+        if ((o1 == null)) {
+            return (o2 == null) ? /*both null*/Integer.valueOf(0) : Integer.valueOf(-1);
         }
-        /*
-         * Both references are not null
-         */
-        return null;
+        return (o2 == null) ? Integer.valueOf(1) : /*both not null*/null;
     }
 
     /**
