@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -380,13 +380,25 @@ public final class IMAPDefaultFolderChecker {
                                     Integer.valueOf(session.getContextId()),
                                     "Waiting time elapsed.");
                             }
-                            f.get();
+                            try {
+                                f.get();
+                            } catch (final ExecutionException e) {
+                                if (MailAccount.DEFAULT_ID == accountId) {
+                                    /*
+                                     * Mandatory for primary mail account
+                                     */
+                                    throw ThreadPools.launderThrowable(e, MailException.class);
+                                }
+                                final MailException me = ThreadPools.launderThrowable(e, MailException.class);
+                                LOG.warn(new StringBuilder(128).append("A default folder could not be created on IMAP server ").append(
+                                    imapConfig.getServer()).append(" for login ").append(imapConfig.getLogin()).append(" (user=").append(
+                                    session.getUserId()).append(", context=").append(session.getContextId()).append("): ").append(
+                                    me.getMessage()).toString(), me);
+                            }
                         }
                     } catch (final InterruptedException e) {
                         // Keep interrupted status
                         throw new MailException(MailException.Code.INTERRUPT_ERROR, e);
-                    } catch (final ExecutionException e) {
-                        throw ThreadPools.launderThrowable(e, MailException.class);
                     }
                     if (DEBUG) {
                         LOG.debug(new StringBuilder(64).append("Default folders check for account ").append(accountId).append(" took ").append(
@@ -522,12 +534,21 @@ public final class IMAPDefaultFolderChecker {
         final StringBuilder tmp = new StringBuilder(32);
         final boolean checkSubscribed = true;
         final long st = System.currentTimeMillis();
-        Folder f = imapStore.getFolder(tmp.append(prefix).append(name).toString());
+        final String fullname = tmp.append(prefix).append(name).toString();
+        Folder f = imapStore.getFolder(fullname);
         tmp.setLength(0);
-        if (!f.exists() /* && !f.create(type) */) {
+        if (!f.exists()) {
             if (isFullname) {
-                /* OK, a fullname was passed. Try to create obviously non-existing IMAP folder. */
-                IMAPCommandsCollection.createFolder((IMAPFolder) f, sep, type);
+                /*
+                 * OK, a fullname was passed. Try to create obviously non-existing IMAP folder.
+                 */
+                try {
+                    IMAPCommandsCollection.createFolder((IMAPFolder) f, sep, type);
+                    return null;
+                } catch (final MessagingException e) {
+                    LOG.warn(new StringBuilder(64).append("Creation of non-existing default IMAP folder \"").append(fullname).append(
+                        "\" failed.").toString(), e);
+                }
             } else {
                 /*
                  * A name was passed. Perform a case-insensitive look-up because some IMAP servers do not allow to create a folder of which
@@ -559,11 +580,23 @@ public final class IMAPDefaultFolderChecker {
                     /*
                      * Zero or more than one candidate found. Try to create IMAP folder
                      */
-                    IMAPCommandsCollection.createFolder((IMAPFolder) f, sep, type);
+                    try {
+                        IMAPCommandsCollection.createFolder((IMAPFolder) f, sep, type);
+                    } catch (final MessagingException e) {
+                        LOG.warn(new StringBuilder(64).append("Creation of non-existing default IMAP folder \"").append(fullname).append(
+                            "\" failed.").toString(), e);
+                    }
                 } else {
                     if (MailAccount.DEFAULT_ID == accountId) {
                         // Must not edit default mail account. Try to create IMAP folder
-                        IMAPCommandsCollection.createFolder((IMAPFolder) f, sep, type);
+                        try {
+                            IMAPCommandsCollection.createFolder((IMAPFolder) f, sep, type);
+                        } catch (final MessagingException e) {
+                            LOG.warn(
+                                new StringBuilder(64).append("Creation of non-existing default IMAP folder \"").append(fullname).append(
+                                    "\" failed.").toString(),
+                                e);
+                        }
                     } else {
                         /*
                          * Found _ONE_ candidate of which name passed ignore-case comparison
