@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,6 +53,8 @@ import static com.openexchange.mail.MailServletInterface.mailInterfaceMonitor;
 import static com.openexchange.mail.dataobjects.MailFolder.DEFAULT_FOLDER_ID;
 import static com.openexchange.mail.mime.utils.MIMEMessageUtility.fold;
 import static com.openexchange.mail.mime.utils.MIMEStorageUtility.getFetchProfile;
+import gnu.trove.TIntObjectHashMap;
+import gnu.trove.TLongIntHashMap;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -78,6 +80,7 @@ import com.openexchange.imap.cache.UserFlagsCache;
 import com.openexchange.imap.command.CopyIMAPCommand;
 import com.openexchange.imap.command.FetchIMAPCommand;
 import com.openexchange.imap.command.FlagsIMAPCommand;
+import com.openexchange.imap.command.NewFetchIMAPCommand;
 import com.openexchange.imap.config.IIMAPProperties;
 import com.openexchange.imap.search.IMAPSearch;
 import com.openexchange.imap.services.IMAPServiceRegistry;
@@ -93,6 +96,7 @@ import com.openexchange.mail.MailFields;
 import com.openexchange.mail.MailPath;
 import com.openexchange.mail.MailSortField;
 import com.openexchange.mail.OrderDirection;
+import com.openexchange.mail.api.IMailMessageStorageExt;
 import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.dataobjects.compose.ComposeType;
@@ -125,7 +129,7 @@ import com.sun.mail.imap.Rights;
  * 
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
-public final class IMAPMessageStorage extends IMAPFolderWorker {
+public final class IMAPMessageStorage extends IMAPFolderWorker implements IMailMessageStorageExt {
 
     private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(IMAPMessageStorage.class);
 
@@ -207,71 +211,75 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
         return imapProperties;
     }
 
+    public MailMessage[] getMessages(final String fullname, final String[] mailIds, final MailField[] mailFields, final String[] headerNames) throws MailException {
+        if ((mailIds == null) || (mailIds.length == 0)) {
+            return EMPTY_RETVAL;
+        }
+        return getMessagesInternal(fullname, uids2longs(mailIds), mailFields, headerNames);
+    }
+
     @Override
     public MailMessage[] getMessagesLong(final String fullname, final long[] mailIds, final MailField[] mailFields) throws MailException {
         if ((mailIds == null) || (mailIds.length == 0)) {
             return EMPTY_RETVAL;
         }
+        return getMessagesInternal(fullname, mailIds, mailFields, null);
+    }
+
+    private MailMessage[] getMessagesInternal(final String fullname, final long[] uids, final MailField[] mailFields, final String[] headerNames) throws MailException {
         final MailFields fieldSet = new MailFields(mailFields);
-        final boolean body;
         /*
          * Check for field FULL
          */
-        if (fieldSet.contains(MailField.FULL)) {
-            final MailMessage[] mails = new MailMessage[mailIds.length];
+        if (fieldSet.contains(MailField.FULL) || fieldSet.contains(MailField.BODY)) {
+            final MailMessage[] mails = new MailMessage[uids.length];
             for (int j = 0; j < mails.length; j++) {
-                mails[j] = getMessageLong(fullname, mailIds[j], false);
+                mails[j] = getMessageLong(fullname, uids[j], false);
             }
             return mails;
         }
         /*
          * Get messages with given fields filled
          */
-        body = fieldSet.contains(MailField.BODY) || fieldSet.contains(MailField.FULL);
         try {
             imapFolder = setAndOpenFolder(imapFolder, fullname, Folder.READ_ONLY);
             /*
              * Fetch desired messages by given UIDs. Turn UIDs to corresponding sequence numbers to maintain order cause some IMAP servers
              * ignore the order of UIDs provided in a "UID FETCH" command.
              */
-            final int[] seqNums = IMAPCommandsCollection.uids2SeqNums(imapFolder, mailIds);
-            final Message[] messages = new Message[seqNums.length];
+            final TLongIntHashMap seqNumsMap = IMAPCommandsCollection.uids2SeqNumsMap(imapFolder, uids);
+            final MailMessage[] messages = new MailMessage[uids.length];
             final MailField[] fields = fieldSet.toArray();
-            final FetchProfile fetchProfile = getFetchProfile(fields, getIMAPProperties().isFastFetch());
+            final FetchProfile fetchProfile = getFetchProfile(fields, headerNames, null, null, getIMAPProperties().isFastFetch());
             final boolean isRev1 = imapConfig.getImapCapabilities().hasIMAP4rev1();
-            int lastPos = 0;
-            int pos = 0;
-            while (pos < seqNums.length) {
-                if (seqNums[pos] <= 0) {
-                    final int len = pos - lastPos;
-                    if (len > 0) {
-                        fetchValidSeqNumsWithFallback(lastPos, len, seqNums, messages, fetchProfile, isRev1, body);
-                    }
-                    // Determine next valid position
-                    pos++;
-                    while (pos < seqNums.length && -1 == seqNums[pos]) {
-                        pos++;
-                    }
-                    lastPos = pos;
-                } else {
-                    pos++;
+            final TIntObjectHashMap<MailMessage> fetchedMsgs =
+                fetchValidSeqNumsWithFallback(seqNumsMap.getValues(), fetchProfile, isRev1, false);
+            /*
+             * Fill array
+             */
+            for (int i = 0; i < uids.length; i++) {
+                final long uid = uids[i];
+                final int seqNum = seqNumsMap.get(uid);
+                if (seqNum > 0) {
+                    messages[i] = fetchedMsgs.get(seqNum);
                 }
             }
-            if (lastPos < pos) {
-                fetchValidSeqNumsWithFallback(lastPos, pos - lastPos, seqNums, messages, fetchProfile, isRev1, body);
-            }
+            /*
+             * Check field existence
+             */
+            MIMEMessageConverter.checkFieldExistence(messages, mailFields);
             if (fieldSet.contains(MailField.ACCOUNT_NAME) || fieldSet.contains(MailField.FULL)) {
-                return setAccountInfo(convert2Mails(messages, fields, body));
+                return setAccountInfo(messages);
             }
-            return convert2Mails(messages, fields, body);
+            return messages;
         } catch (final MessagingException e) {
             throw MIMEMailException.handleMessagingException(e, imapConfig, session);
         }
     }
 
-    private void fetchValidSeqNumsWithFallback(final int lastPos, final int len, final int[] seqNums, final Message[] messages, final FetchProfile fetchProfile, final boolean isRev1, final boolean body) throws MailException, MessagingException {
+    private TIntObjectHashMap<MailMessage> fetchValidSeqNumsWithFallback(final int[] seqNums, final FetchProfile fetchProfile, final boolean isRev1, final boolean body) throws MailException, MessagingException {
         try {
-            fetchValidSeqNums(lastPos, len, seqNums, messages, fetchProfile, isRev1, body, false);
+            return fetchValidSeqNums(seqNums, fetchProfile, isRev1, body, false);
         } catch (final FolderClosedException e) {
             throw MIMEMailException.handleMessagingException(e, imapConfig, session);
         } catch (final StoreClosedException e) {
@@ -280,29 +288,31 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
             if (DEBUG) {
                 LOG.debug("Fetch with BODYSTRUCTURE failed.", e);
             }
-            fetchValidSeqNums(lastPos, len, seqNums, messages, fetchProfile, isRev1, body, true);
+            return fetchValidSeqNums(seqNums, fetchProfile, isRev1, body, true);
         }
     }
 
-    private void fetchValidSeqNums(final int lastPos, final int len, final int[] seqNums, final Message[] messages, final FetchProfile fetchProfile, final boolean isRev1, final boolean body, final boolean ignoreBodystructure) throws MessagingException {
-        final int[] subarr = new int[len];
-        System.arraycopy(seqNums, lastPos, subarr, 0, len);
+    private TIntObjectHashMap<MailMessage> fetchValidSeqNums(final int[] seqNums, final FetchProfile fetchProfile, final boolean isRev1, final boolean body, final boolean ignoreBodystructure) throws MessagingException {
         final long start = System.currentTimeMillis();
-        final Message[] submessages;
+        final MailMessage[] submessages;
         if (ignoreBodystructure) {
             submessages =
-                new FetchIMAPCommand(imapFolder, isRev1, subarr, FetchIMAPCommand.getSafeFetchProfile(fetchProfile), false, true, body).setDetermineAttachmentyHeader(
+                new NewFetchIMAPCommand(imapFolder, isRev1, seqNums, FetchIMAPCommand.getSafeFetchProfile(fetchProfile), false, true, body).setDetermineAttachmentByHeader(
                     true).doCommand();
         } else {
-            submessages = new FetchIMAPCommand(imapFolder, isRev1, subarr, fetchProfile, false, true, body).doCommand();
+            submessages = new NewFetchIMAPCommand(imapFolder, isRev1, seqNums, fetchProfile, false, true, body).doCommand();
         }
         final long time = System.currentTimeMillis() - start;
         mailInterfaceMonitor.addUseTime(time);
         if (DEBUG) {
-            LOG.debug(new StringBuilder(128).append("IMAP fetch for ").append(subarr.length).append(" messages took ").append(time).append(
+            LOG.debug(new StringBuilder(128).append("IMAP fetch for ").append(seqNums.length).append(" messages took ").append(time).append(
                 STR_MSEC).toString());
         }
-        System.arraycopy(submessages, 0, messages, lastPos, submessages.length);
+        final TIntObjectHashMap<MailMessage> ret = new TIntObjectHashMap<MailMessage>(seqNums.length);
+        for (int i = 0; i < seqNums.length; i++) {
+            ret.put(seqNums[i], submessages[i]);
+        }
+        return ret;
     }
 
     @Override
@@ -1755,7 +1765,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
              * Handle ham.
              */
             try {
-                IMAPProvider.getInstance().getSpamHandler().handleHam(
+                SpamHandlerRegistry.getSpamHandlerBySession(session, accountId, IMAPProvider.getInstance()).handleHam(
                     accountId,
                     imapFolder.getFullName(),
                     longs2uids(msgUIDs),
@@ -1870,11 +1880,19 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
     }
 
     private MailMessage[] convert2Mails(final Message[] msgs, final MailField[] fields) throws MailException {
-        return convert2Mails(msgs, fields, false);
+        return convert2Mails(msgs, fields, null, false);
     }
 
     private MailMessage[] convert2Mails(final Message[] msgs, final MailField[] fields, final boolean includeBody) throws MailException {
-        return MIMEMessageConverter.convertMessages(msgs, fields, includeBody);
+        return convert2Mails(msgs, fields, null, includeBody);
+    }
+
+    private MailMessage[] convert2Mails(final Message[] msgs, final MailField[] fields, final String[] headerNames) throws MailException {
+        return convert2Mails(msgs, fields, headerNames, false);
+    }
+
+    private MailMessage[] convert2Mails(final Message[] msgs, final MailField[] fields, final String[] headerNames, final boolean includeBody) throws MailException {
+        return MIMEMessageConverter.convertMessages(msgs, fields, headerNames, includeBody);
     }
 
 }
