@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -472,10 +472,12 @@ public final class FetchIMAPCommand extends AbstractIMAPCommand<Message[]> {
                 FetchItemHandler itemHandler = MAP.get(item.getClass());
                 if (null == itemHandler) {
                     itemHandler = getItemHandlerByItem(item, loadBody);
-                }
-                if (null == itemHandler) {
-                    if (LOG.isWarnEnabled()) {
-                        LOG.warn("Unknown FETCH item: " + item.getClass().getName());
+                    if (null == itemHandler) {
+                        if (LOG.isWarnEnabled()) {
+                            LOG.warn("Unknown FETCH item: " + item.getClass().getName());
+                        }
+                    } else {
+                        itemHandler.handleItem(item, msg, LOG);
                     }
                 } else {
                     itemHandler.handleItem(item, msg, LOG);
@@ -671,7 +673,12 @@ public final class FetchIMAPCommand extends AbstractIMAPCommand<Message[]> {
             }
             for (final Enumeration<?> e = h.getAllHeaders(); e.hasMoreElements();) {
                 final Header hdr = (Header) e.nextElement();
-                msg.addHeader(hdr.getName(), hdr.getValue());
+                final String name = hdr.getName();
+                if (MessageHeaders.HDR_SUBJECT.equals(name)) {
+                    msg.addHeader(name, MIMEMessageUtility.checkNonAscii(hdr.getValue()));
+                } else {
+                    msg.addHeader(name, hdr.getValue());
+                }
                 /*-
                  * 
                 final HeaderHandler hdrHandler = hdrHandlers.get(hdr.getName());
@@ -724,7 +731,8 @@ public final class FetchIMAPCommand extends AbstractIMAPCommand<Message[]> {
             if (null == env.subject) {
                 subject = "";
             } else {
-                final char[] chars = env.subject.toCharArray();
+                final String subj = MIMEMessageUtility.checkNonAscii(env.subject);
+                final char[] chars = subj.toCharArray();
                 final StringBuilder sb = new StringBuilder(chars.length);
                 int i = 0;
                 while (i < chars.length) {
