@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -96,7 +96,8 @@ public final class IMAPConfig extends MailConfig {
 
     @Override
     public MailCapabilities getCapabilities() {
-        return imapCapabilities == null ? MailCapabilities.EMPTY_CAPS : imapCapabilities;
+        final IMAPCapabilities capabilities = imapCapabilities;
+        return capabilities == null ? MailCapabilities.EMPTY_CAPS : capabilities;
     }
 
     /**
@@ -179,10 +180,8 @@ public final class IMAPConfig extends MailConfig {
      */
     public boolean isImapSearch() {
         final boolean imapSearch = IMAPProperties.getInstance().isImapSearch();
-        if (imapCapabilities != null) {
-            return (imapSearch && (imapCapabilities.hasIMAP4rev1() || imapCapabilities.hasIMAP4()));
-        }
-        return imapSearch;
+        final IMAPCapabilities capabilities = imapCapabilities;
+        return (capabilities != null) ? (imapSearch && (capabilities.hasIMAP4rev1() || capabilities.hasIMAP4())) : imapSearch;
     }
 
     /**
@@ -192,10 +191,8 @@ public final class IMAPConfig extends MailConfig {
      */
     public boolean isImapSort() {
         final boolean imapSort = IMAPProperties.getInstance().isImapSort();
-        if (imapCapabilities != null) {
-            return (imapSort && imapCapabilities.hasSort());
-        }
-        return imapSort;
+        final IMAPCapabilities capabilities = imapCapabilities;
+        return (capabilities != null) ? (imapSort && capabilities.hasSort()) : imapSort;
     }
 
     @Override
@@ -215,8 +212,9 @@ public final class IMAPConfig extends MailConfig {
      */
     public boolean isSupportsACLs() {
         final BoolCapVal supportsACLs = IMAPProperties.getInstance().getSupportsACLs();
-        if (imapCapabilities != null && BoolCapVal.AUTO.equals(supportsACLs)) {
-            return imapCapabilities.hasPermissions();
+        final IMAPCapabilities capabilities = imapCapabilities;
+        if (capabilities != null && BoolCapVal.AUTO.equals(supportsACLs)) {
+            return capabilities.hasPermissions();
         }
         return BoolCapVal.TRUE.equals(supportsACLs) ? true : false;
     }
